Test repository to experiment with GitLab [CI components](https://docs.gitlab.com/ee/ci/components/).

[TOC]

## General tips
* The jobs in this CI component are job templates. This means, by default, all job names start with a dot.
  This approach has two advantages: you can use the job to instantiate multiple jobs from the same template using `extends`,
  placing conditions on jobs becomes easier and avoids cases when no job is scheduled.
* Use the `name` argument to use a component directly without `extends`

## Components

### Kaniko build

The `kaniko-build` CI/CD component provides a template to build Docker images within a pipeline using [Kaniko](https://github.com/GoogleContainerTools/kaniko).

#### Parameter

| Parameter           | Description                                                         | Type     | Default                 |
| ------------------- | ------------------------------------------------------------------- | -------- | ----------------------- |
| `dockerfile`        | Path of the Dockerfile relative to the project directory.           | `string` | `Dockerfile`            |
| `image_repo`        | Name of the Docker repository including the registry path           | `string` | `$CI_REGISTRY_IMAGE`    |
| `image_tag`         | Tag of the Docker image                                             | `string` | `$CI_COMMIT_TAG`        |
| `stage`             | Stage of the CI/CD job                                              | `string` | `build`                 |
| `name`              | Name of the CI/CD job                                               | `string` | `.kaniko-build`         |
| `kaniko_tag`        | Kaniko docker image tag. THe job needs debug variants of Kaniko.    | `string` | `v1.20.1-debug`         |
| `registry`          | Hostname (and port) of the Docker registry to which the job logs in | `string` | `$CI_REGISTRY`          |
| `registry_user`     | Username to authenticate with the Docker registry                   | `string` | `$CI_REGISTRY_USER`     |
| `registry_password` | Password to authenticate with the Docker registry                   | `string` | `$CI_REGISTRY_PASSWORD` |

#### Examples

```yaml
stages:
  - build

include:
  - component: gitlab.sauerburger.com/sauerburger-it/devops/kaniko-build@0.1.0

docker-build:
  extends: .kaniko-build
  rules: 
  - if: $CI_COMMIT_TAG

```

### Docker image retag

The `buildx-retag` CI/CD component provides a template to push existing Docker image to the registry using a different tag. The
operation doesn't require the job to download the image artifacts.

#### Parameter

| Parameter           | Description                                                         | Type     | Default                             |
| ------------------- | ------------------------------------------------------------------- | -------- | ----------------------------------- |
| `source`            | Repository and tag of the existing source Docker image.             | `string` | `$CI_REGISTRY_IMAGE:$CI_COMMIT_TAG` |
| `target`            | Repository and tag of the Docker image to push.                     | `string` | `$CI_REGISTRY_IAMGE:latest`         |
| `stage`             | Stage of the CI/CD job                                              | `string` | `deploy`                            |
| `name`              | Name of the CI/CD job                                               | `string` | `.buildx-retag`                     |
| `docker_tag`        | "Docker" Docker image tag.                                          | `string` | `25.0.3-cli`                        |
| `registry`          | Hostname (and port) of the Docker registry to which the job logs in | `string` | `$CI_REGISTRY`                      |
| `registry_user`     | Username to authenticate with the Docker registry                   | `string` | `$CI_REGISTRY_USER`                 |
| `registry_password` | Password to authenticate with the Docker registry                   | `string` | `$CI_REGISTRY_PASSWORD`             |

#### Examples

```yaml
stages:
  - deploy

include:
  - component: gitlab.sauerburger.com/sauerburger-it/devops/buildx-retag@0.1.0

tag-latest:
  extends: .buildx-retag
  rules:
  - if: $CI_COMMIT_TAG
    when: manual
```

### Deploy static Kubernetes manifest


#### Parameter

| Parameter      | Description                                               | Type     | Default                  |
| -------------- | --------------------------------------------------------- | -------- | ------------------------ |
| `agent_name`   | Name of the Kubernetes agent without the repository info. | `string` | *(mandatory)*            |
| `stage`        | Stage of the CI/CD job.                                   | `string` | `deploy`                 |
| `name`         | Name of the CI/CD job                                     | `string` | `.kubernetes-deploy`     |
| `agent_repo`   | Repository info of the Kubernetes agent                   | `string` | Repository path and name |
| `docker_image` | "kubectl" Docker image.                                   | `string` | `alpine/k8s:1.28.3`      |
| `namespace`    | Namespace used in `kubectl apply`                         | `string` | `default`                |
| `manifest`     | Filename of the Kubernetes manifest to `apply`            | `string` | `kubernetes.yaml`        |

#### Examples

```yaml
stages:
  - deploy

include:
  - component: gitlab.sauerburger.com/sauerburger-it/devops/kubernetes-deploy@0.1.0
    inputs:
      agent_name: stratus-agent

kubernetes-deploy:
  extends: .kubernetes-deploy
  rules:
  - if: $CI_COMMIT_TAG
    when: manual

kubernetes-deploy-2:
  extends: .kubernetes-deploy
  variables:
    # Overwrite defaults
    NAMESPACE: other-namespace
    AGENT_NAME: staging-agent
    MANIFEST: staging.yaml
  rules:
  - if: $CI_COMMIT_TAG
    when: manual
```